@@ -35,9 +35,18 @@ extern "C"
 #include <mapnik/image_util.hpp>
 #include <mapnik/jpeg_io.hpp>
 #include <mapnik/png_io.hpp>
+#include <mapnik/image_reader.hpp>
 #include <sstream>
+// cairo
+#if defined(HAVE_CAIRO) && defined(HAVE_PYCAIRO)
+#include <cairomm/surface.h>
+#include <pycairo.h>
+#endif
 
 using mapnik::Image32;
+using mapnik::ImageReader;
+using mapnik::get_image_reader;
+using mapnik::type_from_filename;
 using namespace boost::python;
 using mapnik::save_to_file;
 
@@ -51,26 +60,43 @@ PyObject* tostring1( Image32 const& im)
 // encode (png,jpeg)
 PyObject* tostring2(Image32 const & im, std::string const& format)
 {
-   std::ostringstream ss(std::ios::out|std::ios::binary);
-   if (format == "png") save_as_png(ss,im.data());
-   else if (format == "png256") save_as_png256(ss,im.data());
-   else if (format == "jpeg") save_as_jpeg(ss,85,im.data());
-   else throw mapnik::ImageWriterException("unknown format: " + format);
-   return ::PyString_FromStringAndSize((const char*)ss.str().c_str(),ss.str().size());
+   std::string s = save_to_string(im, format);
+   return ::PyString_FromStringAndSize(s.data(),s.size());
 }
 
 void (*save_to_file1)( mapnik::Image32 const&, std::string const&,std::string const&) = mapnik::save_to_file;
 void (*save_to_file2)( mapnik::Image32 const&, std::string const&) = mapnik::save_to_file;
 
+boost::shared_ptr<Image32> open_from_file(std::string const& filename)
+{
+   std::auto_ptr<ImageReader> reader(get_image_reader(filename,type_from_filename(filename)));
+   if (reader.get())
+   {
+      boost::shared_ptr<Image32> image_ptr(new Image32(reader->width(),reader->height()));
+      reader->read(0,0,image_ptr->data());
+      return image_ptr;
+   }
+   throw mapnik::ImageReaderException("FIXME: " + filename);  
+}
+
 void blend (Image32 & im, unsigned x, unsigned y, Image32 const& im2, float opacity)
 {
    im.set_rectangle_alpha2(im2.data(),x,y,opacity);
 }
 
+#if defined(HAVE_CAIRO) && defined(HAVE_PYCAIRO)
+boost::shared_ptr<Image32> from_cairo(PycairoSurface* surface)
+{
+    Cairo::RefPtr<Cairo::ImageSurface> s(new Cairo::ImageSurface(surface->surface));
+    boost::shared_ptr<Image32> image_ptr(new Image32(s));
+    return image_ptr;
+}
+#endif
+
 void export_image()
 {
     using namespace boost::python;
-    class_<Image32>("Image","This class represents a 32 bit RGBA image.",init<int,int>())
+    class_<Image32,boost::shared_ptr<Image32> >("Image","This class represents a 32 bit RGBA image.",init<int,int>())
        .def("width",&Image32::width)
        .def("height",&Image32::height)
        .def("view",&Image32::get_view)
@@ -82,5 +108,12 @@ void export_image()
        .def("tostring",&tostring2)
        .def("save", save_to_file1)
        .def("save", save_to_file2)
+       .def("open",open_from_file)
+       .staticmethod("open")
+#if defined(HAVE_CAIRO) && defined(HAVE_PYCAIRO)
+       .def("from_cairo",&from_cairo)
+       .staticmethod("from_cairo")
+#endif
        ;    
+    
 }
