@@ -34,7 +34,7 @@
 namespace mapnik
 {
     MAPNIK_DECL void load_map(Map & map, std::string const& filename, bool strict = false);
-    MAPNIK_DECL void load_map_string(Map & map, std::string const& str, bool strict = false);
+    MAPNIK_DECL void load_map_string(Map & map, std::string const& str, bool strict = false, std::string const& base_url="");
 }
 
 #endif // LOAD_MAP_HPP
