@@ -48,12 +48,6 @@
 #include <sstream>
 #include <iomanip>
 
-#ifndef MAPNIK_BIG_ENDIAN
-#define WKB_ENCODING "NDR"
-#else
-#define WKB_ENCODING "XDR"
-#endif
-
 #define FMAX std::numeric_limits<double>::max()
 
 DATASOURCE_PLUGIN(postgis_datasource)
@@ -74,6 +68,7 @@ using mapnik::attribute_descriptor;
 postgis_datasource::postgis_datasource(parameters const& params)
    : datasource(params),
      table_(*params_.get<std::string>("table","")),
+     schema_(""),
      geometry_table_(*params_.get<std::string>("geometry_table","")),
      geometry_field_(*params_.get<std::string>("geometry_field","")),
      cursor_fetch_size_(*params_.get<int>("cursor_size",0)),
@@ -154,11 +149,10 @@ postgis_datasource::postgis_datasource(parameters const& params)
          {
              geometry_table_ = table_from_sql(table_);
          }
-         std::string schema_name="";
          std::string::size_type idx = geometry_table_.find_last_of('.');
          if (idx!=std::string::npos)
          {
-            schema_name = geometry_table_.substr(0,idx);
+            schema_ = geometry_table_.substr(0,idx);
             geometry_table_ = geometry_table_.substr(idx+1);
          }
          else
@@ -179,8 +173,8 @@ postgis_datasource::postgis_datasource(parameters const& params)
              s << "SELECT f_geometry_column, srid FROM ";
              s << GEOMETRY_COLUMNS <<" WHERE f_table_name='" << unquote(geometry_table_) <<"'";
              
-             if (schema_name.length() > 0) 
-                s << " AND f_table_schema='" << unquote(schema_name) << "'";
+             if (schema_.length() > 0) 
+                s << " AND f_table_schema='" << unquote(schema_) << "'";
             
              if (geometry_field_.length() > 0)
                 s << " AND f_geometry_column='" << unquote(geometry_field_) << "'";
@@ -473,14 +467,14 @@ featureset_ptr postgis_datasource::features(const query& q) const
          if (!geometryColumn_.length() > 0)
          {
              std::ostringstream s_error;
-             s_error << "PostGIS: geometry name lookup failed for table '" << geometry_table_
+             s_error << "PostGIS: geometry name lookup failed for table '" << schema_ << "." << geometry_table_
              << "'. Please manually provide the 'geometry_field' parameter or add an entry "
-             << "in the geometry_columns for '" << geometry_table_ << "'.";
+             << "in the geometry_columns for '" << schema_ << "." << geometry_table_ << "'.";
              throw mapnik::datasource_exception(s_error.str());
          }
 
          std::ostringstream s;
-         s << "SELECT AsBinary(\"" << geometryColumn_ << "\",'" << WKB_ENCODING << "') AS geom";
+         s << "SELECT AsBinary(\"" << geometryColumn_ << "\") AS geom";
          std::set<std::string> const& props=q.property_names();
          std::set<std::string>::const_iterator pos=props.begin();
          std::set<std::string>::const_iterator end=props.end();
@@ -520,13 +514,13 @@ featureset_ptr postgis_datasource::features_at_point(coord2d const& pt) const
          if (!geometryColumn_.length() > 0)
          {
              std::ostringstream s_error;
-             s_error << "PostGIS: geometry name lookup failed for table '" << geometry_table_
+             s_error << "PostGIS: geometry name lookup failed for table '" << schema_ << "." << geometry_table_
              << "'. Please manually provide the 'geometry_field' parameter or add an entry "
-             << "in the geometry_columns for '" << geometry_table_ << "'.";
+             << "in the geometry_columns for '" << schema_ << "." << geometry_table_ << "'.";
              throw mapnik::datasource_exception(s_error.str());
          }
                     
-         s << "SELECT AsBinary(\"" << geometryColumn_ << "\",'" << WKB_ENCODING << "') AS geom";
+         s << "SELECT AsBinary(\"" << geometryColumn_ << "\") AS geom";
             
          std::vector<attribute_descriptor>::const_iterator itr = desc_.get_descriptors().begin();
          std::vector<attribute_descriptor>::const_iterator end = desc_.get_descriptors().end();
@@ -567,45 +561,60 @@ Envelope<double> postgis_datasource::envelope() const
       {
          PoolGuard<shared_ptr<Connection>,shared_ptr<Pool<Connection,ConnectionCreator> > > guard(conn,pool);
          std::ostringstream s;
-         boost::optional<std::string> estimate_extent = params_.get<std::string>("estimate_extent");
+         boost::optional<mapnik::boolean> estimate_extent = params_.get<mapnik::boolean>("estimate_extent",false);
 
          if (!geometryColumn_.length() > 0)
          {
              std::ostringstream s_error;
              s_error << "PostGIS: unable to query the layer extent of table '"
-             << geometry_table_ << "' because we cannot determine the geometry field name."
+             << schema_ << "." << geometry_table_ << "' because we cannot determine the geometry field name."
              << "\nPlease provide either 1) an 'extent' parameter to skip this query, "
              << "2) a 'geometry_field' and/or 'geometry_table' parameter, or 3) add a "
              << "record to the 'geometry_columns' for your table.";
              throw mapnik::datasource_exception(s_error.str());
          }
-         // TODO - do we need to respect schema here?
-         if (estimate_extent && *estimate_extent == "true")
+
+         if (estimate_extent && *estimate_extent)
          {
-            s << "select xmin(ext),ymin(ext),xmax(ext),ymax(ext)"
-              << " from (select estimated_extent('" 
-              << geometry_table_ << "','" 
-              << geometryColumn_ << "') as ext) as tmp";
+             s << "select xmin(ext),ymin(ext),xmax(ext),ymax(ext)"
+               << " from (select estimated_extent('";
+
+             if (schema_.length() > 0)
+             {
+                 s << unquote(schema_) << "','";
+             }
+
+             s << unquote(geometry_table_) << "','"
+               << unquote(geometryColumn_) << "') as ext) as tmp";
          }
-         else 
+         else
          {
             s << "select xmin(ext),ymin(ext),xmax(ext),ymax(ext)"
               << " from (select extent(" <<geometryColumn_<< ") as ext from ";
               if (extent_from_subquery_)
-                  // if a subselect limits records then calculating the extent upon the 
+              {
+                  // if a subselect limits records then calculating the extent upon the
                   // subquery will be faster and the bounds will be more accurate
                   s << populate_tokens(table_) << ") as tmp";
+              }
               else
+              {
+                  if (schema_.length() > 0)
+                  {
+                      s << schema_ << ".";
+                  }
+
                   // but if the subquery does not limit records then querying the
                   // actual table will be faster as indexes can be used
                   s << geometry_table_ << ") as tmp";
+              }
          }
 
          /*if (show_queries_)
          {
              clog << boost::format("PostGIS: sending query: %s\n") % s.str();
          }*/
-                     
+            
          shared_ptr<ResultSet> rs=conn->executeQuery(s.str());
          if (rs->next())
          {
