@@ -26,37 +26,33 @@
 #include <mapnik/ellipsoid.hpp>
 #include <cmath>
 
-// When using Sun's C++ compiler, use the `std` namespace to get the math routines.
-#ifdef __SUNPRO_CC
-using std::atan2;
-using std::cos;
-using std::pow;
-using std::sin;
-using std::sqrt;
-#endif
-
 namespace mapnik {
     
-    static const double deg2rad = 0.0174532925199432958;
-    static const double R = 6372795.0; // average great-circle radius of the earth
-    
-    double great_circle_distance::operator() (coord2d const& pt0, 
-                                              coord2d const& pt1) const
-    {
-        double lon0 = pt0.x * deg2rad;
-        double lat0 = pt0.y * deg2rad;
-        double lon1 = pt1.x * deg2rad;
-        double lat1 = pt1.y * deg2rad;
-        
-        double dlat = lat1 - lat0;
-        double dlon = lon1 - lon0;
-        
-        double sin_dlat = sin(0.5 * dlat);
-        double sin_dlon = sin(0.5 * dlon);
-        
-        double a = pow(sin_dlat,2.0) + cos(lat0)*cos(lat1)*pow(sin_dlon,2.0);
-        double c = 2 * atan2(sqrt(a),sqrt(1 - a));
-        return R * c; 
-    }
-    
+   using std::atan2;
+   using std::cos;
+   using std::pow;
+   using std::sin;
+   using std::sqrt;
+
+   static const double deg2rad = 0.0174532925199432958;
+   static const double R = 6372795.0; // average great-circle radius of the earth
+   
+   double great_circle_distance::operator() (coord2d const& pt0, 
+                                             coord2d const& pt1) const
+   {
+      double lon0 = pt0.x * deg2rad;
+      double lat0 = pt0.y * deg2rad;
+      double lon1 = pt1.x * deg2rad;
+      double lat1 = pt1.y * deg2rad;
+      
+      double dlat = lat1 - lat0;
+      double dlon = lon1 - lon0;
+      
+      double sin_dlat = sin(0.5 * dlat);
+      double sin_dlon = sin(0.5 * dlon);
+      
+      double a = pow(sin_dlat,2.0) + cos(lat0)*cos(lat1)*pow(sin_dlon,2.0);
+      double c = 2 * atan2(sqrt(a),sqrt(1 - a));
+      return R * c; 
+   }
 }
