@@ -239,14 +239,17 @@ namespace mapnik
 #endif
     Envelope<T> Envelope<T>::intersect(const EnvelopeType& other) const
     {
+        if (intersects(other)) {
+            T x0=std::max(minx_,other.minx_);
+            T y0=std::max(miny_,other.miny_);
 
-        T x0=std::max(minx_,other.minx_);
-        T y0=std::max(miny_,other.miny_);
+            T x1=std::min(maxx_,other.maxx_);
+            T y1=std::min(maxy_,other.maxy_);
 
-        T x1=std::min(maxx_,other.maxx_);
-        T y1=std::min(maxy_,other.maxy_);
-
-        return Envelope<T>(x0,y0,x1,y1);
+            return Envelope<T>(x0,y0,x1,y1);
+        } else {
+            return Envelope<T>();
+        }
     }
 
     template <typename T>
@@ -304,21 +307,27 @@ namespace mapnik
     template <typename T>    
     Envelope<T>& Envelope<T>::operator*=(T t)
     {
-        minx_ *= t;
-        miny_ *= t;
-        maxx_ *= t;
-        maxy_ *= t;
-        return *this;
+       coord<T,2> c = center();
+       T sx = static_cast<T>(0.5 * width()  * t);
+       T sy = static_cast<T>(0.5 * height() * t);
+       minx_ = c.x - sx;
+       maxx_ = c.x + sx;
+       miny_ = c.y - sy;
+       maxy_ = c.y + sy;
+       return *this;
     }
-
+   
     template <typename T>    
     Envelope<T>& Envelope<T>::operator/=(T t)
     {
-        minx_ /= t;
-        miny_ /= t;
-        maxx_ /= t;
-        maxy_ /= t;
-        return *this;
+       coord<T,2> c = center();
+       T sx = static_cast<T>(0.5 * width() / t);
+       T sy = static_cast<T>(0.5 * height() / t);
+       minx_ = c.x - sx;
+       maxx_ = c.x + sx;
+       miny_ = c.y - sy;
+       maxy_ = c.y + sy;
+       return *this;
     }
     
     template class Envelope<int>;
