@@ -25,9 +25,14 @@
 
 namespace mapnik
 {
-    filter_ptr create_filter (std::string const& wkt)
-    {
-        filter_factory<Feature> factory;
-        return factory.compile(wkt);
-    }
+   filter_ptr create_filter (std::string const& wkt,std::string const& encoding)
+   {
+      transcoder tr(encoding);
+      return filter_factory<Feature>::compile(wkt,tr);
+   }
+   
+   filter_ptr create_filter (std::string const& wkt)
+   {
+      return create_filter(wkt,"utf8");
+   }
 }
