@@ -38,8 +38,9 @@ namespace mapnik {
 #ifdef MAPNIK_THREADSAFE
         mutex::scoped_lock lock(projection::mutex_);
 #endif
-        is_source_latlong_ = pj_is_latlong(source_.proj_);
-        is_dest_latlong_ = pj_is_latlong(dest_.proj_);
+        is_source_latlong_ = pj_is_latlong(source_.proj_) ? true : false;
+        is_dest_latlong_ = pj_is_latlong(dest_.proj_) ? true : false ;
+        is_source_equal_dest = (source_ == dest_);
     }
     
     bool proj_transform::forward (double & x, double & y , double & z) const
@@ -47,6 +48,9 @@ namespace mapnik {
 #ifdef MAPNIK_THREADSAFE
         mutex::scoped_lock lock(projection::mutex_);
 #endif
+        if (is_source_equal_dest)
+            return true;
+
         if (is_source_latlong_)
         {
             x *= DEG_TO_RAD;
@@ -73,6 +77,8 @@ namespace mapnik {
 #ifdef MAPNIK_THREADSAFE
         mutex::scoped_lock lock(projection::mutex_);
 #endif
+        if (is_source_equal_dest)
+            return true;
       
         if (is_dest_latlong_)
         {
@@ -93,5 +99,15 @@ namespace mapnik {
         }
         
         return true;
-    } 
+    }
+
+    mapnik::projection const& proj_transform::source() const
+    {
+        return source_;
+    }
+    mapnik::projection const& proj_transform::dest() const
+    {
+        return dest_;
+    }
+ 
 }
